// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef MESSAGE_PAYLOAD_H
#define MESSAGE_PAYLOAD_H

#include <stdint.h>

#include <istream>
#include <ostream>
#include <vector>

//-----------------------------------------------------------------------------
// MessagePayload
//-----------------------------------------------------------------------------

/**
 * @brief This class represents the contents of a network message.
 *
 * It provides means to encode and to decode data to network format (i.e. deal
 * with endianness), and also can be treated like an ordinary array.
 */
class MessagePayload
{
public:
    explicit MessagePayload( const std::size_t &payload_size_in_bytes );
    MessagePayload( const MessagePayload& other );
    ~MessagePayload();

    MessagePayload& operator=( const MessagePayload &other );

    const uint8_t& operator[]( const std::size_t offset ) const;
    uint8_t& operator[]( const std::size_t offset );

    uint8_t* get();

    void append(
            const uint8_t *extra_payload_data,
            const std::size_t extra_payload_size_in_bytes
    );

    bool decode1(
            const int byte_index,
            const int bit_index
    ) const;
    void encode1(
            const int byte_index,
            const int bit_index,
            const bool value
    );

    uint16_t decode16(
            const int left_byte_index,
            const int right_byte_index
    ) const;
    void encode16(
            const int left_byte_index,
            const int right_byte_index,
            const uint16_t value
    );

    uint32_t decode32(
            const int first_byte,
            const int last_byte
    ) const;
    void encode32(
            const int first_byte,
            const int last_byte,
            const uint32_t value
    );

    std::istream& read( std::istream &is );
    std::ostream& write( std::ostream &os ) const;

private:
    /// The size of the payload buffer
    std::size_t PayloadSizeInBytes;
    /// The payload buffer
    std::vector<uint8_t> Payload;

};

#endif // MESSAGE_PAYLOAD_H
